#include <stdio.h>
#include <stdlib.h>

#include "ActionStack.h"
#include "FileIO/Parser.h"

#include "Debug/Debug.h"

static char *PushVar  (Parser &p, char *str, ActionStack *a);
static char *PushQuote(Parser &,  char *str, ActionStack *a);

Vars::Vars(void)
{
}

Vars::Vars(const CString &src, const CString &obj, const CString &incdirs, const CString &libs, const CString &title)
{
  Source      = src;
  ObjFile     = obj;
  IncludeDirs = incdirs;
  Libs        = libs;
  Title       = title;
}


Action::Action(int cmd, char *str)
{
  Command = cmd;
  if(str)
    Str = str;
}

void Action::Run(CString &str, const Vars &v)
{
  switch(Command)
  {
    case CMD_STR:
      str += Str;
      break;

    case CMD_OBJ_FILE:
      str += '\"';
      str += v.ObjFile;
      str += '\"';
      break;

    case CMD_SRC_FILE:
      str += '\"';
      str += v.Source;
      str += '\"';
      break;

    case CMD_COMPILED_OBJS:
      str += v.ObjFile;
      break;

    case CMD_INCLUDE_DIRS:
      str += v.IncludeDirs;
      break;

    case CMD_FILE_TITLE:
      remove(v.Title.c_str());
      str += '\"';
      str += v.Title;
      str += '\"';
      break;

    case CMD_LIBS:
      str += v.Libs;
      break;
  }
}

void ActionStack::Expand(const CString &str)
{
  char *buffer;
  Parser p(0);

  Cmds.clear();

  if(!str.size())
    return;

  p.LoadMemory(str.c_str());

  p.AddTokenSeparator("%",    0,  1, 0, (PCBACK)PushVar,   this);
  p.AddTokenSeparator("\\\"", 0, -1, 0, (PCBACK)PushQuote, this);

  p.AddTokenSet();
  p.AddTokenSeparator("%", 0, 0);

  p.SetTokenSet(0);

  for(buffer = p.GrabToken(); buffer; buffer = p.GrabToken())
  {
    Cmds.push_back(Action(CMD_STR, buffer));
    free(buffer);
  }
}

void ActionStack::Run(const Vars &v)
{
  unsigned int i;
  CString cmd;

  for(i = 0; i < Cmds.size(); ++i)
    Cmds[i].Run(cmd, v);

  //printf("Running: |%s|\n", cmd.c_str());
  fflush(stdout);
  system(cmd.c_str());
  fflush(stdout);
}

char *PushVar(Parser &p, char *str, ActionStack *a)
{
  char *var = p.GrabToken();

  VALIDATE_POINTER(a)

  if(Cmp(var, "OBJ"))
    a->Cmds.push_back(Action(CMD_OBJ_FILE));
  else
  if(Cmp(var, "SOURCE"))
    a->Cmds.push_back(Action(CMD_SRC_FILE));
  else
  if(Cmp(var, "COMPILED OBJS"))
    a->Cmds.push_back(Action(CMD_COMPILED_OBJS));
  else
  if(Cmp(var, "INCLUDE DIRS"))
    a->Cmds.push_back(Action(CMD_INCLUDE_DIRS));
  else
  if(Cmp(var, "COMPILED LIBS"))
    a->Cmds.push_back(Action(CMD_LIBS));
  else
  if(Cmp(var, "BIN"))
    a->Cmds.push_back(Action(CMD_FILE_TITLE));

  free(var);

  return str;
}

char *PushQuote(Parser &, char *str, ActionStack *a)
{
  VALIDATE_POINTER(a)

  a->Cmds.push_back(Action(CMD_STR, "\""));
  return str;
}
