#include <stdio.h>

#include "FileIO/Parser.h"
#include "Database.h"

#include "Debug/Debug.h"

Database::~Database(void)
{
  unsigned int i;
  for(i = 0; i < Files.size(); ++i)
    delete Files[i];
}

void Database::Read(Parser &p)
{
  File *f = new File;

  while(f->Read(p))
  {
    Files.push_back(f);
    f = new File;
  }

  delete f;
}

void Database::Write(FILE *fp)
{
  unsigned int i;
  
  #if CMM_ENABLED | SAFEALLOC_ENABLED
    {for(unsigned int i = 0; i < Files.size(); ++i)
      VALIDATE_POINTER(Files[i])}
  #endif
  
  fprintf(fp, "[DATABASE]\n");
  for(i = 0; i < Files.size(); ++i)
    if(Files[i]->WasModified)
    {
      if(!Files[i]->Checksum)
        Files[i]->GetChecksum();
      if(!Files[i]->LastModified)
        Files[i]->GetTimeModifed();
      Files[i]->Write(fp);
    }
}

void Database::CheckTree(Directory *tree)
{
  unsigned int i;

  if(!tree)
    return;

  VALIDATE_POINTER(tree)

  #if CMM_ENABLED | SAFEALLOC_ENABLED
    for(unsigned int i = 0; i < tree->Headers.size(); ++i)
      VALIDATE_POINTER(tree->Headers[i])
    for(unsigned int i = 0; i < tree->Sources.size(); ++i)
      VALIDATE_POINTER(tree->Sources[i])
  #endif

  for(i = 0; i < tree->Headers.size(); ++i)
    tree->Headers[i]->WasModified = FileModified(tree->Headers[i]);

  for(i = 0; i < tree->Sources.size(); ++i)
    tree->Sources[i]->WasModified = FileModified(tree->Sources[i]);
}

int Database::FileModified(File *file)
{
  unsigned int i;
  
  VALIDATE_POINTER(file)
  #if CMM_ENABLED | SAFEALLOC_ENABLED
    {for(unsigned int i = 0; i < Files.size(); ++i)
      VALIDATE_POINTER(Files[i])}
  #endif

  for(i = 0; i < Files.size(); ++i)
    if(*Files[i] == *file)
    {
      Files[i]->WasModified = 1;
      if(file->Modified(*Files[i]))
      {
        Files[i]->Checksum     = file->Checksum;
        Files[i]->LastModified = file->LastModified;
        return 1;
      }
      return 0;
    }

  AddFile(file);
  return 1;
}

void Database::Clean(void)
{
  unsigned int i;
  
  #if CMM_ENABLED | SAFEALLOC_ENABLED
    {for(unsigned int i = 0; i < Files.size(); ++i)
      VALIDATE_POINTER(Files[i])}
  #endif

  for(i = 0; i < Files.size(); ++i)
    delete Files[i];
  Files.clear();
}

void Database::AddFile(const File *f)
{
  VALIDATE_POINTER((void *)f)
  Files.push_back(new File());
  Files.back()->Name = f->Name;
  Files.back()->Path = f->Path;
  Files.back()->WasModified = 1;
}
