#ifndef _CRT_SECURE_NO_WARNINGS
  #define _CRT_SECURE_NO_WARNINGS
  #define _CRT_SECURE_NO_DEPRECATE
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

#define ERROR_OUTPUT_OVERLOAD_PRINTF 0
#include "ErrorOutput.h"

#define WRITE_TO_CONSOLE (1 << 0)
#define WRITE_TO_FILE    (1 << 1)
#define OVERWRITE_FILE   (1 << 2)
#define ADD_TIMESTAMP    (1 << 3)

FILE        *FP    = 0;
const char  *File  = 0;
unsigned int Flags = WRITE_TO_CONSOLE | ADD_TIMESTAMP;




static void OpenFile(const char *file);
static void WriteTimeStamp(const char *str);
static void WriteFileHeader(void);


void DebugWriteToConsole(bool state)
{
  if(state)
    Flags |=  WRITE_TO_CONSOLE;
  else
    Flags &= ~WRITE_TO_CONSOLE;
}

void DebugWriteToFile(bool state, const char *file)
{
  if(state)
    Flags |=  WRITE_TO_FILE;
  else
    Flags &= ~WRITE_TO_FILE;

  OpenFile(file);
}

void DebugOverwriteFile(bool state)
{
  if(state)
    Flags |=  OVERWRITE_FILE;
  else
    Flags &= ~OVERWRITE_FILE;

  OpenFile(File);
}

void DebugAddTimestamp(bool state)
{
  if(state)
    Flags |=  ADD_TIMESTAMP;
  else
    Flags &= ~ADD_TIMESTAMP;
}

int Debugprintf(const char *format, ...)
{
  bool append_new_line = false;
  char write[5 * 1024];
  int  written;
  va_list arg;

  if(!(Flags & (WRITE_TO_CONSOLE | WRITE_TO_FILE)))
    return 0;

  va_start(arg, format);
  written = vsprintf(write, format, arg);
  va_end(arg);

  if(written)
    append_new_line = write[written - 1] != '\n' && write[written - 1] != '\r';

  if(FP)
  {
    if(Flags & ADD_TIMESTAMP)
      WriteTimeStamp(write);
    else
      fwrite(write, 1, written, FP);
    if(append_new_line)
      fwrite("\n", 1, 1, FP);
    fflush(FP);
  }

  if(Flags & WRITE_TO_CONSOLE)
    printf("%s", write);

  return written;
}

void ErrorOutputCleanup(void)
{
  if(FP)
  {
    fwrite("\n\n", 1, 2, FP);
    fclose(FP);
  }
  FP = 0;
}

void OpenFile(const char *file)
{
  if(FP)
    fclose(FP);

  if(!file || !file[0] || !(Flags & WRITE_TO_FILE))
  {
    FP   = 0;
    File = 0;
    return;
  }

  if(Flags & OVERWRITE_FILE)
    FP = fopen(file, "w");
  else
    FP = fopen(file, "a");

  File = file;

  if(FP)
    WriteFileHeader();
}

void WriteTimeStamp(const char *str)
{
  unsigned int start, end;
  int          pass = 0;
  time_t       rawtime;
  struct tm   *timeinfo;
  char         buffer[80];

  time(&rawtime);
  timeinfo = localtime(&rawtime);

  strftime(buffer, sizeof(buffer) / sizeof(buffer[0]), "%I:%M:%S: ",timeinfo);

  fprintf(FP, "%s", buffer);

  end = ~(unsigned int)0;
  do
  {
    for(end = start = end + 1; str[end] && str[end] != '\n' && str[end] != '\r'; ++end)
      ;

    if(end - start)
    {
      if(pass)
        fwrite("        : ", 1, 10, FP);
      fwrite(str + start, 1, end - start, FP);
      fwrite("\n", 1, 1, FP);
      pass = 1;
    }
  }
  while(str[end]);
}

void WriteFileHeader(void)
{
  time_t       rawtime;
  struct tm   *timeinfo;
  char         weekday[10];
  char         month[10];
  char         dayyear[15];

  time(&rawtime);
  timeinfo = localtime(&rawtime);

  strftime(weekday, sizeof(weekday) / sizeof(weekday[0]), "%A",     timeinfo);
  strftime(month,   sizeof(month)   / sizeof(month[0]),   "%B",     timeinfo);
  strftime(dayyear, sizeof(dayyear) / sizeof(dayyear[0]), "%d, %Y", timeinfo);

  fprintf(FP, "/-----------------------------\\\n");
  fprintf(FP, "|%9s, %9s %s|\n", weekday, month, dayyear);
  fprintf(FP, "\\-----------------------------/\n");
}
