/*******************************************************************************
Note: Please append MM_START("file to write errors to") and
        MM_END("file to write leaks to") to the main(), like so

        int main(void)
        {
           MM_START("errors.txt")
             // Your code here...
           MM_END("leaks.txt")
          return 0;
        }

All the following settings should be defined to either 0 (Disable) or 1 (Enable)

  General Config Defines
ENABLE_CMM          - When enabled, the CMM functions will be able. This
                        includes handling access violations. If this is not
                        enabled, all CMM function will not be available, and
                        access violations can still occur.
ENABLE_SAFEALLOC    - When enabled, additional debugging related functions will
                        be enabled. This includes detection of
                          Double delete
                          delete garbage
                          underflow corruption detection
                          overflow corruption detection (Not via page access)
                          *alloc(), new, delete mismatch
                          basic memory profiling
                          various other stats
OVERLOAD_C_ALLOC    - When enabled, malloc(), calloc(), realloc(), and free()
                        will be overloaded via a define. This will automatically
                        be resolved to use the SafeAlloc *alloc(). If SafeAlloc
                        is not enabled, and CMM is enabled, then the CMM version
                        of *alloc() will be used.
OVERLOAD_CPP_ALLOC  - When enabled, all the new and delete functions will be
                        overloaded. Like the C allocs, this will automatically
                        be resolved to the correct version of new, based on the
                        state of ENABLE_CMM and ENABLE_SAFEALLOC
MM_BREAK_ON_ERROR   - Causes SafeAlloc to break on a detected error, and
                        determines if CMM adds a padding of no access memory
                        to each pointer.
                      If disabled, no padding will be appended to the end of a
                        allocated block of memory, so memory corruption will be
                        possible, even if CMM is enabled. To fight this, it is
                        highly recommend you enable SafeAlloc, and define a
                        acceptable padding area after a allocation. This will
                        allow SafeAlloc to detect corruption; however, it will
                        not know where the corruption came from.

  Output related defines - Please add MM_START("file") to the top of main()
MM_WRITE_TO_CONSOLE - Should the errors be written to the console
                        (uses printf()) on a error, or not? This settings is
                        recommended, unless you have other debug information
                        being dumped to the console.
MM_WRITE_TO_FILE    - Should the errors be written to a file? If enabled, all
                        errors will be written to the file specified by
                        MM_START("file to write errors to.txt")
                        which should be at the top of main().
MM_OVERWRITE_FILE   - Should the file specified by MM_START("file") be
                        overwritten, or appended to? If set to 0, the file will
                        be appended to if it exists.

  Output At End - Please add MM_END("file") to the bottom of main()
MM_AT_END_WRITE_TO_CONSOLE    - Should the leaks be dumped to the console?
MM_AT_END_WRITE_LEAKS_TO_FILE - Should the leaks be dumped to a file?
MM_AT_END_OVERWRITE_FILE      - Should the file be overwritten?

  SafeAlloc Config Defines
PADDING_BEFORE      - How many bytes to append to the start of a block of
                        memory, in order to allow SafeAlloc to detect
                        underwrites. This is transparent to the rest of the
                        code. You do not have to perform any manual offsetting.
PADDING_AFTER       - How many bytes to append to the end of a block of memory.
                        This allows SafeAlloc to detect corruption after a
                        block of memory, IE overflows. This will not enabled
                        SafeAlloc to detect access violations; however, it will
                        notify you of the corruption.
                      If CMM is enabled, it is highly recommended that you set
                        this value to 0. If this value is not 0, the access
                        violations will not occur until the SafeAlloc padding
                        has been exceeded.
PADDINGBYTE         - Byte code to write to the padding bytes.
ALLOCSIZE           - Initial size of SafeAllocs hash table. The larger the
                        value, the lower number of growths that will be
                        required, in exchange for a high memory footprint.
DUMPSTRING          - Attempt to dump the pointers as strings? This setting
                        can be extremely dangerous, since it does not do
                        bounding checking. This is only meant for dealing
                        with raw string processing.
                      It is highly recommended you set this value to 0.
ACTIVECHECK         - Check all valid pointers padding area for corruption
                        when ever possible. This includes during all
                        allocations, deallocations, and any other calls to
                        SafeAlloc.
                      This will greatly effect program performance; however,
                        it can be extremely useful for detecting corruption
                        as soon as possible, when the CMM is disabled.
FORCEFREE           - When enabled, if you free a garbage pointer, or a pointer
                        that was not allocated through SafeAlloc, the pointer
                        will still be deleted.
                      It is highly recommended you set this value to 0. This
                        should only be enabled if you want to attempt to
                        reproduce free list corruption.
*******************************************************************************/
#ifndef MEMORY_DEBUGGER_H
#define MEMORY_DEBUGGER_H

#define DEBUG_WINDOWS                 0

#define ENABLE_CMM                    0
#define ENABLE_SAFEALLOC              0
#define OVERLOAD_C_ALLOC              0
#define OVERLOAD_CPP_ALLOC            0

  /* MM_START() Configuration */
#define MM_BREAK_ON_ERROR             1

  /* SafeAlloc Specific Defines */
#define PADDING_BEFORE               32
#define PADDING_AFTER                 0
#define PADDINGBYTE                0xAA
#define ALLOCSIZE                  1024
#define DUMPSTRING                    0
#define ACTIVECHECK                   0
#define FORCEFREE                     0



















/******************************************************************************
 * Helper defines below. Ignore.                                              *
 ******************************************************************************/

#if !DEBUG_WINDOWS
  #ifdef CMM_ENABLED
    #undef CMM_ENABLED
  #endif
  #undef  ENABLE_CMM
  #define ENABLE_CMM 0
#endif

#if ENABLE_CMM
  #ifndef CMM_ENABLED
    #define CMM_ENABLED 1
  #endif
#else
  #ifndef CMM_ENABLED
    #define CMM_ENABLED 0
  #endif
#endif

#if ENABLE_SAFEALLOC
  #ifndef SAFEALLOC_ENABLED
    #define SAFEALLOC_ENABLED 1
  #endif
#else
  #ifndef SAFEALLOC_ENABLED
    #define SAFEALLOC_ENABLED 0
  #endif
#endif

#if OVERLOAD_C_ALLOC
  #ifndef CMM_OVERLOAD_C_ALLOC
    #define CMM_OVERLOAD_C_ALLOC CMM_ENABLED
  #endif
  #ifndef SAFEALLOC_OVERLOAD_C_ALLOC
    #define SAFEALLOC_OVERLOAD_C_ALLOC SAFEALLOC_ENABLED
  #endif
#else
  #ifndef CMM_OVERLOAD_C_ALLOC
    #define CMM_OVERLOAD_C_ALLOC 0
  #endif
  #ifndef SAFEALLOC_OVERLOAD_C_ALLOC
    #define SAFEALLOC_OVERLOAD_C_ALLOC 0
  #endif
#endif


#if OVERLOAD_CPP_ALLOC
  #ifndef CMM_OVERLOAD_CPP_ALLOC
    #define CMM_OVERLOAD_CPP_ALLOC CMM_ENABLED
  #endif
  #ifndef SAFEALLOC_OVERLOAD_CPP_ALLOC
    #define SAFEALLOC_OVERLOAD_CPP_ALLOC SAFEALLOC_ENABLED
  #endif
#else
  #ifndef CMM_OVERLOAD_CPP_ALLOC
    #define CMM_OVERLOAD_CPP_ALLOC 0
  #endif
  #ifndef SAFEALLOC_OVERLOAD_CPP_ALLOC
    #define SAFEALLOC_OVERLOAD_CPP_ALLOC 0
  #endif
#endif

#if SAFEALLOC_OVERLOAD_C_ALLOC && CMM_OVERLOAD_C_ALLOC
  #undef  CMM_OVERLOAD_C_ALLOC
  #define CMM_OVERLOAD_C_ALLOC 0
#endif

#if SAFEALLOC_OVERLOAD_CPP_ALLOC && CMM_OVERLOAD_CPP_ALLOC
  #undef  CMM_OVERLOAD_CPP_ALLOC
  #define CMM_OVERLOAD_CPP_ALLOC 0
#endif

#ifndef ERROR_OUTPUT_OVERLOAD_PRINTF
  #define ERROR_OUTPUT_OVERLOAD_PRINTF 0
#endif

#include "CMM.h"
#include "SafeAlloc.h"
#include "ErrorOutput.h"

#if SAFEALLOC_ENABLED
  #define IS_A_BASE_POINTER(a) sIsABasePointer(a)
#elif CMM_ENABLED
  #define IS_A_BASE_POINTER(a) CMMIsABasePointer(a)
#else
  #define IS_A_BASE_POINTER(a) ((a) != 0)
#endif

#if SAFEALLOC_ENABLED
  #define IS_A_MEM_POINTER(a) sIsAValidPointer(a)
#elif CMM_ENABLED
  #define IS_A_MEM_POINTER(a) CMMIsAValidPointer(a)
#else
  #define IS_A_MEM_POINTER(a) ((a) != 0)
#endif

#if SAFEALLOC_ENABLED | CMM_ENABLED
  #define VALIDATE_POINTER(a) {if(!IS_A_MEM_POINTER((void *)(a))) {printf("Error: %35s: %4d: Invalid Pointer: %p\n", __FILE__, __LINE__, (void *)(a));fflush(stdout);}}
#else
  #define VALIDATE_POINTER(a)
#endif

#if SAFEALLOC_ENABLED
  #define POINTER_SIZE(a) sPointerSize(a);
#elif CMM_ENABLED
  #define POINTER_SIZE(a) CMMPointerSize(a);
#else
  #define POINTER_SIZE(a) 0
#endif

#if MM_BREAK_ON_ERROR
  #if ENABLE_SAFEALLOC && ENABLE_CMM
    #define MM_BREAK_ON_ERROR__ {CMMEnablePadding(); sBreakOnError(1);}
  #elif ENABLE_SAFEALLOC
    #define MM_BREAK_ON_ERROR__ {sBreakOnError(1);}
  #elif ENABLE_CMM
    #define MM_BREAK_ON_ERROR__ {CMMEnablePadding();}
  #else
    #define MM_BREAK_ON_ERROR__
  #endif
#else
  #if ENABLE_SAFEALLOC && ENABLE_CMM
    #define MM_BREAK_ON_ERROR__ {CMMDisablePadding(); sBreakOnError(0);}
  #elif ENABLE_SAFEALLOC
    #define MM_BREAK_ON_ERROR__ {sBreakOnError(0);}
  #elif ENABLE_CMM
    #define MM_BREAK_ON_ERROR__ {CMMDisablePadding();}
  #else
    #define MM_BREAK_ON_ERROR__
  #endif
#endif

#define MM_START() MM_BREAK_ON_ERROR__

#endif
