#define _CRT_SECURE_NO_DEPRECATE
#define _CRT_SECURE_NO_WARNINGS

#include <stdio.h>
#include <stdlib.h>

#include "../OS Specific/DirList.h"
#include "../OS Specific/SystemCalls.h"
#include "../FileIO/Parser.h"
#include "../ConfigFile.h"

#include "Management.h"
#include "../OS Specific/DirString.h"
#include "Directory.h"
#include "FilePaths.h"
#include "File.h"
#include "ParserCache.h"

#include "../Debug/Debug.h"

static ParserCache *PCache = 0;

File::File(void)
{
  if(!PCache) PCache = new ParserCache(); PCache->AddRef();

  WasModified  = 0;
  LastModified = 0;
  Checksum     = 0;
}

File::File(const CString &path, const CString &file, Directory *root, ConfigFile *config)
{
  if(!PCache) PCache = new ParserCache(); PCache->AddRef();

  Path   = path;
  Name   = file;
  Root   = root;
  Config = config;

  WasModified = 0;

  GetTimeModifed();
  Checksum = 0;
  //GetChecksum();
}

File::~File(void)
{
  PCache = PCache->RemoveRef();
}

void File::CheckIncludes(int istemp)
{
  CVector<CString> includes;
  char *buffer;

  VALIDATE_POINTER(Config)

  Parser *p = PCache->GetIncludeParser();
  p->LoadFile((Path + Name).c_str());

  for(buffer = p->GrabToken(); buffer; buffer = p->GrabToken())
  {
    if(p->GetParams() == (void *)1)
    {
      CString dir;
      CString name;

      free(buffer);

      buffer = p->GrabToken();
      if(!buffer || p->GetParams() != (void *)2)
      {
        free(buffer);
        continue;
      }

      File *f = FindFile(buffer);
      if(f)
      {
        VALIDATE_POINTER(f)
        VALIDATE_POINTER(f->Config)
        if(!istemp)
          Config->AddDependency(this, f, f->Config);
        if(f->WasModified)
        {
          WasModified = 1;
          SetModified();
        }
        if(!istemp)
          f->IncludedBy.push_back(this);
      }
      else
        if(CheckAdditonalIncludesDirs(buffer))
          SetModified();
    }

    free(buffer);
  }
  
  PCache->ReleaseIncludeParser();
  
  #if SAFEALLOC_ENABLED
    RenameAlloc(IncludedBy.mData, "Included By List");
  #endif
}

void File::SetModified(void)
{
  unsigned int i;
  WasModified = 2;

  #if CMM_ENABLED | SAFEALLOC_ENABLED
    for(unsigned int i = 0; i < IncludedBy.size(); ++i)
      VALIDATE_POINTER(IncludedBy[i])
  #endif

  for(i = 0; i < IncludedBy.size(); ++i)
    if(IncludedBy[i]->WasModified != 2)
      IncludedBy[i]->SetModified();

  WasModified = 1;
}

File *File::FindFile(const CString &name)
{
  File *f;
  CString path = GetRelativeLocation(name);
  CString file = GetFile(name);

  if((f = FindFile(Root, path, file)) != 0)
    return f;

  return FindFile(Root, file);
}

File *File::FindFile(Directory *node, const CString &dir, const CString &file)
{
  unsigned int i;
  File *f;
  
  VALIDATE_POINTER(node);

  if(node->Path == dir)
  {
    #if CMM_ENABLED | SAFEALLOC_ENABLED
     {for(unsigned int i = 0; i < node->Headers.size(); ++i)
        VALIDATE_POINTER(node->Headers[i])
      for(unsigned int i = 0; i < node->Sources.size(); ++i)
        VALIDATE_POINTER(node->Sources[i])}
    #endif
  
    for(i = 0; i < node->Headers.size(); ++i)
      if(node->Headers[i]->Name == file)
        return node->Headers[i];

    for(i = 0; i < node->Sources.size(); ++i)
      if(node->Sources[i]->Name == file)
        return node->Sources[i];
  }

  #if CMM_ENABLED | SAFEALLOC_ENABLED
    for(unsigned int i = 0; i < node->Dirs.size(); ++i)
      VALIDATE_POINTER(node->Dirs[i])
  #endif

  for(i = 0; i < node->Dirs.size(); ++i)
    if((f = FindFile(node->Dirs[i], dir, file)) != 0)
      return f;

  return 0;
}

File *File::FindFile(Directory *node, const CString &file)
{
  unsigned int i;
  File *f;
  
  VALIDATE_POINTER(node);
  #if CMM_ENABLED | SAFEALLOC_ENABLED
   {for(unsigned int i = 0; i < node->Headers.size(); ++i)
      VALIDATE_POINTER(node->Headers[i])
    for(unsigned int i = 0; i < node->Sources.size(); ++i)
      VALIDATE_POINTER(node->Sources[i])
    for(unsigned int i = 0; i < node->Dirs.size(); ++i)
      VALIDATE_POINTER(node->Dirs[i])}
  #endif

  for(i = 0; i < node->Headers.size(); ++i)
    if(node->Headers[i]->Name == file)
      return node->Headers[i];

  for(i = 0; i < node->Sources.size(); ++i)
    if(node->Sources[i]->Name == file)
      return node->Sources[i];

  for(i = 0; i < node->Dirs.size(); ++i)
    if((f = FindFile(node->Dirs[i], file)) != 0)
      return f;

  return 0;
}

CString File::GetRelativeLocation(const CString &name)
{
  DirString str(Path);
  str.Dir += GetPath(name);
  str.Validate();

  return str.Dir;
}

int File::CheckAdditonalIncludesDirs(const CString &buffer)
{
  unsigned int i;
  CString file = GetRelativeLocation(buffer) + GetFile(CString(buffer));

  VALIDATE_POINTER(Root)
  VALIDATE_POINTER(Root->Config)

  if(FileExists(file))
    return Root->Config->CheckDataBase(file, Root);

  for(i = 0; i < Root->Config->AdditonalIncludeDirs.size(); ++i)
  {
    file = Root->Config->AdditonalIncludeDirs[i] + buffer;
    if(FileExists(file))
      return Root->Config->CheckDataBase(file, Root);
  }

  return 0;
}

void File::GetTimeModifed(void)
{
  LastModified = GetModTime((Path + Name).c_str());
}

void File::GetChecksum(void)
{
  char *buffer;
  int i, j;
  int token = 0;

  Parser *p = PCache->GetChecksumParser();
  
  p->LoadFile((Path + Name).c_str());

  Checksum = 0;

  for(buffer = p->GrabToken(); buffer; buffer = p->GrabToken())
  {
    for(i = 0, j = 0; buffer[i]; ++i, ++j)
    {
      Checksum += (((buffer[i] & 0xAA) << 23) + ((buffer[i] & 0x55) << 13) + buffer[i]) << (j + token);

      if(j >= 7)
        j -= 7;
    }

    free(buffer);
    if(++token >= 13)
      token -= 7;
  }
}

void File::Print(void)
{
  printf("%25s %28s %11d %11d\n", Name.c_str(), Path.c_str(), LastModified, Checksum);
}

void File::Write(FILE *fp)
{
  CString name = "\"";
  CString path;

  if(!Path.size())
    path = "0";
  else
  {
    path  = "\"";
    path += Path;
    path += '\"';
  }

  name += Name;
  name += '\"';

  fprintf(fp, "%25s %28s %11d %11d\n", name.c_str(), path.c_str(), LastModified, Checksum);
}

int File::Read(Parser &p)
{
  int error = 0;
  char *buffer;

  buffer = p.GrabToken();
  if(buffer)
  {
    Name = buffer;
    free(buffer);
  }

  buffer = p.GrabToken();
  if(buffer)
  {
    if(Cmp(buffer, "0"))
      Path = "";
    else
      Path = buffer;
    free(buffer);
  }

  LastModified = p.GrabInt();
  error |= p.ErrorCode();
  Checksum     = p.GrabInt();

  return !error;
}

int File::Modified(File &r)
{
  if(!LastModified)     GetTimeModifed();
  if(!r.LastModified) r.GetTimeModifed();

  if(LastModified != r.LastModified)
  {
    if(!Checksum)     GetChecksum();
    if(!r.Checksum) r.GetChecksum();

    return Checksum != r.Checksum;
  }

  return 0;
}

CString File::GetObjFile(void)
{
  VALIDATE_POINTER(Root)
  VALIDATE_POINTER(Root->Config)
  VALIDATE_POINTER(Config)

  DirString str(Root->Config->ObjDir);
  str.PushDir(Path);
  str.Dir += Name;
  if(Config->Comp)
  {
    VALIDATE_POINTER(Config->Comp)
    str.Dir += Config->Comp->ObjExt;
  }
  else
    str.Dir += ".notset";
  return str.Dir;
}

int File::CompileRequired(void)
{
  VALIDATE_POINTER(Root)
  VALIDATE_POINTER(Root->Config)

  if(!Root->Config->MinRebuild || WasModified || !FileExists(GetObjFile()))
    return 1;

  return 0;
}

bool File::operator==(const CString &r) const
{
  return Name == r;
}

bool File::operator==(const File &r) const
{
  return Name == r.Name && Path == r.Path;
}
