/*******************************************************************************
filename      FilePaths.cpp
author        Craig Williams

Brief Description:

      All content (c) 2008 DigiPen (USA) Corporation, all rights reserved.
*******************************************************************************/
#include <stdlib.h> // malloc()
#include <string.h> // strlen()

#include "FilePaths.h"

#include "../Debug/Debug.h"

char *GetPath(const char *f)
{
  int len = (int)strlen(f);
  char *buffer;
  int i;

  for(i = len; i >= 0 && f[i] != '\\' && f[i] != '/'; --i)
    ;

  if(i < 0)
  {
    buffer = (char *)malloc(1);
    buffer[0] = 0;
  }
  else
  {
    buffer = (char *)malloc(i + 2);
    memcpy(buffer, f, i + 1);
    buffer[i + 1] = 0;
  }

  return buffer;
}

CString GetPath(const CString &f)
{
  CString ret;
  char *buffer = GetPath(f.c_str());
  if(buffer)
  {
    ret = buffer;
    free(buffer);
  }
  return ret;
}

char *GetFile(const char *f)
{
  int len = (int)strlen(f);
  char *buffer;
  int i;

  for(i = len; i >= 0 && f[i] != '\\' && f[i] != '/'; --i)
    ;

  if(i < 0)
  {
    buffer = (char *)malloc(len + 1);
    memcpy(buffer, f, len + 1);
  }
  else
  {
    buffer = (char *)malloc(len - i);
    memcpy(buffer, f + i + 1, len - i);
  }

  return buffer;
}

CString GetFile(const CString &f)
{
  CString ret;
  char *buffer = GetFile(f.c_str());
  if(buffer)
  {
    ret = buffer;
    free(buffer);
  }
  return ret;
}

char *SetExt(const char *file, const char *ext)
{
  int elen = (int)strlen(ext);
  int len  = (int)strlen(file);
  char *buffer;
  int i;

  for(i = len; i >= 0 && file[i] != '.'; --i)
    ;

  if(i < 0)
  {
    buffer = (char *)malloc(len + elen + 2); // +2 - '.' and NULL terminator
    memcpy(buffer, file, len);
    buffer[len] = '.';
    memcpy(buffer + len + 1, ext, elen + 1); // +1 to copy NULL terminator
  }
  else
  {
    buffer = (char *)malloc(i + elen + 2);
    memcpy(buffer, file, i + 1);
    memcpy(buffer + i + 1, ext, elen +1);
  }

  return buffer;
}

char *GetExt(const char *file)
{
  int   len  = (int)strlen(file);
  char *buffer;
  int   i;

  for(i = len; i >= 0 && file[i] != '.'; --i)
    ;

  if(file[i] != '.')
    return 0;

  buffer = (char *)malloc(len - i + 1);
  memcpy(buffer, file + i + 1, len - i);
  buffer[len - i] = 0;
  return buffer;
}

CString GetExt(const CString &f)
{
  CString ret;
  char *buffer = GetExt(f.c_str());
  if(buffer)
  {
    ret = buffer;
    free(buffer);
  }
  return ret;
}

char *CombineStr(const char *dir, const char *file)
{
  int ldir  = (int)strlen(dir);
  int lfile = (int)strlen(file) + 1; // +1 to copy NULL terminator.
  int i;
  char *buffer = (char *)malloc(ldir + lfile);

  for(i = 0; i < ldir; ++i)
    buffer[i] = dir[i];

  for(i = 0; i < lfile; ++i)
    buffer[ldir + i] = file[i];

  return buffer;
}
