#include "ParserCache.h"

#include "../Debug/Debug.h"

ParserCache::ParserCache(void)
{
  Refs         = 0;
  FileChecksum = 0;
  InUse        = 0;
  FileChecksum = 0;
}

ParserCache::~ParserCache(void)
{
  unsigned int i;

  delete FileChecksum;

  for(i = 0; i < IncludeCache.size(); ++i)
    delete IncludeCache[i];
}

void ParserCache::AddRef(void)
{
  ++Refs;
}

ParserCache *ParserCache::RemoveRef(void)
{
  if(!--Refs)
  {
    delete this;
    return 0;
  }

  return this;
}

Parser *ParserCache::GetIncludeParser(void)
{
  if(InUse >= IncludeCache.size())
    BuildIncludeParser();

  IncludeCache[InUse]->SetTokenSet(0);
  return IncludeCache[InUse++];
}

void ParserCache::ReleaseIncludeParser(void)
{
  --InUse;
}

Parser *ParserCache::GetChecksumParser(void)
{
  if(!FileChecksum)
    BuildChecksumParser();
  FileChecksum->SetTokenSet(0);
  return FileChecksum;
}

void ParserCache::BuildChecksumParser(void)
{
  Parser *p = new Parser(0, 1024 * 10); // 10 KB

  p->Enable(PARSER_CONST_TOKEN_SEPS);

  p->AddTokenSeparator(" ",  0);                       // Remove Spaces
  p->AddTokenSeparator("\r", 0);                       // Remove Carriage Ret
  p->AddTokenSeparator("\n", 0);                       // Remove New lines
  p->AddTokenSeparator("\t", 0);                       // Remove Tabs

  p->AddTokenSeparator("/*", 0, 1, 0, GenericDiscard); // Handle C comments
  p->AddTokenSeparator("//", 0, 2, 0, GenericDiscard); // Handle C++ comments
  p->AddTokenSeparator("\"\"", 1);                     // Handle ""
  p->AddTokenSeparator("\"", 1, 3);                    // Handle strings

  p->AddTokenSet(); // 1 - Handle C style comments
  p->AddTokenSeparator("*/", 0, 0);

  p->AddTokenSet(); // 2 - Handle C++ style comments
  p->AddTokenSeparator("\n", 0, 0);
  p->AddTokenSeparator("\r", 0, 0);

  p->AddTokenSet(); // 3 - Handle Strings
  p->AddTokenSeparator("\"", 1, 0);

  p->SetTokenSet(0);

  p->Enable(PARSER_HASH);
  FileChecksum = p;
}

Parser *ParserCache::BuildIncludeParser(void)
{
  Parser *p = new Parser(0, 1024 * 10); // 10 KB

  p->Enable(PARSER_CONST_TOKEN_SEPS);

    // Throw out white space
  p->AddTokenSeparator(" ",  0);
  p->AddTokenSeparator("\n", 0);
  p->AddTokenSeparator("\r", 0);
  p->AddTokenSeparator("\t", 0);

    // Throw out invalid locations that #include could be found
  p->AddTokenSeparator("\"", 0, 1, 0, GenericDiscard); // Throw out strings
  p->AddTokenSeparator("//", 0, 2, 0, GenericDiscard); // Throw out c++ comments
  p->AddTokenSeparator("/*", 0, 3, 0, GenericDiscard); // Throw our c comments

    // Actually handle #include
  p->AddTokenSeparator("#include", 1, 4, 0, 0, (void *)1);

    // TSet 1 - Handle Strings
  p->AddTokenSet();
  p->AddTokenSeparator("\\\\", 0, -1, 1);
  p->AddTokenSeparator("\\\"", 0, -1, 1);
  p->AddTokenSeparator("\"",   0, -2);

    // TSet 2 - Handle C++ comments
  p->AddTokenSet();
  p->AddTokenSeparator("\\\n\r", 0, -1, 1);
  p->AddTokenSeparator("\\\r\n", 0, -1, 1);
  p->AddTokenSeparator("\\\n",   0, -1, 1);
  p->AddTokenSeparator("\\\r",   0, -1, 1);
  p->AddTokenSeparator("\n",     0, -2);
  p->AddTokenSeparator("\r",     0, -2);

    // TSet 3 - Handle C Comments
  p->AddTokenSet();
  p->AddTokenSeparator("*/", 0, -2);

    // TSet 4 - Handle #include
  p->AddTokenSet();
  p->AddTokenSeparator("\"", 0, 5, 0, 0, (void *)2);
  p->AddTokenSeparator("<",  0, 6, 0, 0, (void *)3);
  p->AddTokenSeparator(" ",  0);
  p->AddTokenSeparator("\n", 0);
  p->AddTokenSeparator("\r", 0);
  p->AddTokenSeparator("\t", 0);
  p->AddTokenSeparator("//", 0, 2, 0, GenericDiscard); // Throw out c++ comments
  p->AddTokenSeparator("/*", 0, 3, 0, GenericDiscard); // Throw our c comments

    // TSet 5 - Handle #include Strings
  p->AddTokenSet();
  p->AddTokenSeparator("\"", 0, 0);

    // TSet 6 - Handle Strings
  p->AddTokenSet();
  p->AddTokenSeparator(">", 0, 0);

  p->SetTokenSet(0);

  p->Enable(PARSER_HASH);

  IncludeCache.push_back(p);
  return p;
}
