/*******************************************************************************
filename      DirList.h
author        Craig Williams

Brief Description:
  Provides a system indepenent interface to scan directories for other
    directories and files.

  To open a directory, pass in the desired directoy to the constructor, or
    call OpenDir(). The directoy string that is passed in will be formated
    to the correct format. IE, a string like this
    "//\/\foo/./.\bar/.."
    will become
    "foo/"

  class DirList:
    OpenDir()     - Opens up a directory for scanning. This function is
                      automatically called.
    CloseDir()    - Closes a directory, when you are done with it. Again, this
                      is automatically called.
    GetCurrentDir()-Returns the formatted version of the string passed into
                      the constructor or OpenDir().
    GetDirs()     - Requests the first director from a directory.
    GetNextDir()  - Requests the first and each sequential dir from the current
                      directory.
    GetDirList()  - Returns a list of all the directories in the current
                      directory, as a vector of strings.
    GetFiles()    - Requests the first file from the current directory.
    GetNextFile() - Requests the first and each sequential file from the
                      current directory.
    GetFileList() - Returns a vector of strings that contains the names of all
                      the files in the directory.

  Functions:
    CreateDir()  - Creates a directory.
    DeleteDir()  - Deletes a directory. This is provided since <stdio.h>
                    remove() does not have to delete a dir.
    GetModTime() - Retrieves the last modification time of the specified file.

      All content (c) 2009 DigiPen (USA) Corporation, all rights reserved.
*******************************************************************************/
#ifndef DIR_LIST_H
#define DIR_LIST_H

#include "../Containers/CString.h"
#include "../Containers/CVector.h"

class DirList
{
  public:
    // Current Directory ""
    // Sub Directory "foo/"
    DirList(const CString &dir = "");
    ~DirList(void);

    void OpenDir(const CString &dir);
    void CloseDir(void);

    CString GetCurrentDir(void);

    CString GetDirs(void);
    CString GetNextDir(void);
    CVector<CString> GetDirList(void);
    CString GetFiles(const CString &search);
    CString GetNextFile(const CString &search = "*");
    CVector<CString> GetFileList(const CString &search);

  private:
    void *Params;
    CString Dir;
};

#endif
