/*******************************************************************************
filename      DirString.cpp
author        Craig Williams

Brief Description:

      All content (c) 2009 DigiPen (USA) Corporation, all rights reserved.
*******************************************************************************/

#include "DirString.h"

#include "../Debug/Debug.h"

DirString::DirString(const CString &str)
{
  Dir = str;
  Validate();
}

void DirString::Validate(void)
{
  unsigned int i;
  unsigned int size = Dir.size();

  if(size && (Dir[0] == '/' || Dir[0] == '\\' || Dir[0] == ' '||Dir[0] == '\t'))
  {
    Dir.erase(0, 1);
    --size;
  }

  while(size && (Dir[size - 1] == ' ' || Dir[size - 1] == '\t'))
    Dir.erase(--size, 1);

  if(size && Dir[size - 1] != '/' && Dir[size - 1] != '\\')
  {
    Dir += '/';
    ++size;
  }

  for(i = 1; i < size; ++i)
    if(Dir[i] == '\\')
      Dir[i] = '/';


  for(i = 1; i < size; ++i)
    if(Dir[i] == '/')
      if(!i || Dir[i - 1] == '/')
      {
        Dir.erase(i, 1);
        --size;
        if(i)
          --i;
        --i;
      }
      else
      if(Dir[i - 1] == '.')
        if(i >= 2 && Dir[i - 2] == '.')
        {
          // ../ - Pop back a directory
          int j = i - 4;
          while(j >= 0 && Dir[j] != '/')
            --j;

          if(j == -1 && Dir.size() && Dir[0] != '.')
          {
            j = 0;
            ++i;
          }

          if(j >= 0)
          {
            Dir.erase(j, i - j);
            size -= i - j;
            i    = j;
          }
        }
        else
        {
          // ./ - Current Direcotry
          Dir.erase(i - 2, 2);
          size -= 2;
          i    -= 2;;
        }
}

void DirString::PopDir(void)
{
    // Dir should be in the format "Foo/Bar/"
    // This function will walk from the right to the left unilt a slash is
    //   found.
  int i = Dir.size() - 2;
  CString dir(Dir);

  for(; i >= 0; --i)
    if((dir[i] == '\\' || dir[i] == '/') && dir[i - 1] != '.')
    {
      ++i;
      dir[i] = 0;
      break;
    }

  if(i < 0)
  {
    if(Dir.size() && Dir[0] != '.')
      Dir.clear();
    Dir += "../";
  }
  else
    Dir = dir.c_str();
}

void DirString::PushDir(const CString &str)
{
  Dir += str;
  Dir += '/';
  Validate();
}
