/*******************************************************************************
filename      DirList_Win32.cpp
author        Craig Williams

Brief Description:

      All content (c) 2009 DigiPen (USA) Corporation, all rights reserved.
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <windows.h>

#include "../DirList.h"
#include "../DirString.h"

#include "../../Debug/Debug.h"

DirList::DirList(const CString &dir)
{
  Params = 0;
  OpenDir(dir);
}

DirList::~DirList(void)
{
  CloseDir();
}

void DirList::OpenDir(const CString &_dir)
{
  DirString dir(_dir);
  Dir = dir.Dir;
  CloseDir();
}

void DirList::CloseDir(void)
{
  if(Params)
  {
    FindClose((HANDLE)Params);
    Params = 0;
  }
}

CString DirList::GetCurrentDir(void)
{
  return Dir;
}

CString DirList::GetDirs(void)
{
  WIN32_FIND_DATA file;
  Params = (HANDLE)FindFirstFile((Dir + '*').c_str(), &file);
  if(Params == INVALID_HANDLE_VALUE)
  {
    Params = 0;
    return "";
  }

  return GetNextDir();
}

CString DirList::GetNextDir(void)
{
  WIN32_FIND_DATA file;

  if(!Params)
    return GetDirs();

  while((FindNextFile((HANDLE)Params, &file)))
  {
    if(!strcmp(file.cFileName, ".") || !strcmp(file.cFileName, ".."))
      continue;
    if(file.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
      return file.cFileName;
  }

  CloseDir();
  return "";
}

CVector<CString> DirList::GetDirList(void)
{
  CVector<CString> list;
  CString file;

  while((file = GetNextDir()).length())
    list.push_back(file);

  return list;
}

CString DirList::GetFiles(const CString &search)
{
  WIN32_FIND_DATA file;
  Params = (HANDLE)FindFirstFile((Dir + search).c_str(), &file);
  if(Params == INVALID_HANDLE_VALUE)
  {
    Params = 0;
    return "";
  }

  if(!strcmp(file.cFileName, ".") || !strcmp(file.cFileName, ".."))
    return GetNextFile();

  return file.cFileName;
}

CString DirList::GetNextFile(const CString &search)
{
  WIN32_FIND_DATA file;

  if(!Params)
    return GetFiles(search);

  while((FindNextFile((HANDLE)Params, &file)))
  {
      // Discard "." and ".."
    if(file.cFileName[0] == '.')
      if(!file.cFileName[1] || (file.cFileName[1] == '.' && !file.cFileName[2]))
        continue;

    return file.cFileName;
  }

  CloseDir();
  return "";
}

CVector<CString> DirList::GetFileList(const CString &search)
{
  CVector<CString> list;
  CString file;

  while((file = GetNextFile(search)).length())
    list.push_back(file);

  return list;
}
/*
int main(void)
{
  CString file;

  {
    DirList dir("");
    file = dir.GetDirs();
    printf("Dirs:\n");
    while(file.length())
    {
      printf("    %s\n", file.c_str());
      file = dir.GetNextDir();
    }
  }

  {
    DirList dir("");
    file = dir.GetFiles("*.cpp");
    printf("Files:\n");
    while(file.length())
    {
      printf("    %s\n", file.c_str());
      file = dir.GetNextFile();
    }
  }
}
*/
